package notepad;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * TextFileUtility: It contains utility functions for text file.
 * 
 * @author Neeraj Vishwakarma
 * @version 1.0.0
 * @since 2019-06-11
 */
public class TextFileUtility {
	/**
	 * Create Text File: Function used to create new text file.
	 * 
	 * @param filePath
	 *            is string and it contains location of text file.
	 * @throws IOException
	 *             if file already exist at specified location.
	 */
	public static void createTextFile(String filePath) throws IOException {

		File fp = new File(filePath);
		if (!fp.exists()) {
			if (!fp.getParentFile().exists()) {
				fp.getParentFile().mkdirs();
			}
			fp.createNewFile();
		} else {
			throw (new IOException("File " + filePath + " already exist "));
		}
	}

	/**
	 * Create Text File: Function used to create new text file.
	 * 
	 * @param file
	 *            object passed as a parameter.
	 * @see java.io.File
	 * @throws IOException
	 *             if file already exist at specified location.
	 */
	public static void createTextFile(File file) throws IOException {
		if (!file.exists()) {
			if (!file.getParentFile().exists()) {
				file.getParentFile().mkdirs();
			}
			file.createNewFile();
		} else {
			throw (new IOException("File " + file.getAbsolutePath() + " already exist "));
		}
	}

	/**
	 * Read Text File: Used to read text file.
	 * 
	 * @param filePath
	 *            is string object and it contains location of text file.
	 * @throws IOException
	 *             if file does not exists.
	 * @return <code> List<String> </code>
	 */
	public static List<String> readTextFile(String filePath) throws IOException {
		// use BufferedReader to read text file line by line
		FileReader fReader = null;
		BufferedReader bfrReader = null;
		// Store lines in a
		List<String> lines = new ArrayList<>();
		File fp = new File(filePath);
		if (fp.exists()) {
			fReader = new FileReader(fp);
			bfrReader = new BufferedReader(fReader);
			String line = null;
			do {
				// Read one line and store it in String variable
				line = bfrReader.readLine();

				if (line != null) {
					lines.add(line);
				}

			} while (line != null);

			bfrReader.close();
			fReader.close();

		} else {
			throw (new IOException("File " + filePath + " does not exist."));

		}
		return lines;
	}

	/**
	 * Read Text File: Used to read text file.
	 * 
	 * @param file
	 *            object used as parameter.
	 * @throws IOException
	 *             if file does not exists.
	 * @return <code> List<String> </code>
	 */
	public static List<String> readTextFile(File file) throws IOException {
		// use BufferedReader to read text file line by line
		FileReader fReader = null;
		BufferedReader bfrReader = null;
		// Store lines in a
		List<String> lines = new ArrayList<>();

		if (file.exists()) {
			fReader = new FileReader(file);
			bfrReader = new BufferedReader(fReader);
			String line = null;
			do {
				// Read one line and store it in String variable
				line = bfrReader.readLine();

				if (line != null) {
					lines.add(line);
				}

			} while (line != null);

			bfrReader.close();
			fReader.close();

		} else {
			throw (new IOException("File " + file.getAbsolutePath() + " does not exist."));
		}
		return lines;
	}

	/**
	 * Read Text File: Append new line to existing text file.
	 * 
	 * @param filePath
	 *            is string object and it contains location of text file.
	 * @param line
	 *            is string to append existing text file.
	 * @throws IOException
	 *             if file does not exists.
	 */
	public static void appendLine(String filePath, String line) throws IOException {
		// read old text file and store lines
		List<String> lines = readTextFile(filePath);
		// add new line to the list
		lines.add(line);
		FileWriter fWriter;
		BufferedWriter bWriter;
		File fp = new File(filePath);
		if (fp.exists()) {
			fWriter = new FileWriter(fp);
			bWriter = new BufferedWriter(fWriter);
			// write to text file
			for (String oneLine : lines) {
				bWriter.append(oneLine);
				bWriter.newLine();
			}
			bWriter.close();
			fWriter.close();

		} else {
			throw (new IOException("File " + filePath + " does not exist."));
		}
	}

	/**
	 * Read Text File: Append new line to existing text file.
	 * 
	 * @param file
	 *            object used to pass.
	 * @param line
	 *            is string to append existing text file.
	 * @throws IOException
	 *             if file does not exists.
	 */
	public static void appendLine(File file, String line) throws IOException {
		// read old text file and store lines
		List<String> lines = readTextFile(file);
		// add new line to the list
		lines.add(line);
		FileWriter fWriter;
		BufferedWriter bWriter;

		if (file.exists()) {
			fWriter = new FileWriter(file);
			bWriter = new BufferedWriter(fWriter);
			// write to text file
			for (String oneLine : lines) {
				bWriter.append(oneLine);
				bWriter.newLine();
			}
			bWriter.close();
			fWriter.close();

		} else {
			throw (new IOException("File " + file.getAbsolutePath() + " does not exist."));
		}
	}

	/**
	 * Compare Text File: Compare two text files and also consider or ignore case.
	 * 
	 * @param fSrc
	 *            is first text file.
	 * @param fCmp
	 *            is second text file.
	 * @param ignoreCase
	 *            is boolean value to/not to ignore case.
	 * @throws IOException
	 *             if file size not same or file dosen't exist.
	 * @return <code> boolean </code> as true if both files are same otherwise it
	 *         will return false.
	 */
	public static boolean compareTextFiles(File fSrc, File fCmp, boolean ignoreCase) throws IOException {
		if (fSrc.isFile() && fCmp.isFile()) {
			String fSrcStr = new String(Files.readAllBytes(fSrc.toPath()));
			String fCmpStr = new String(Files.readAllBytes(fCmp.toPath()));

			if (fSrcStr.length() == fCmpStr.length()) {
				if (ignoreCase) {
					return fSrcStr.equalsIgnoreCase(fCmpStr);

				} else {
					return fSrcStr.matches(fCmpStr);
				}
			} else {

				throw (new IOException("File size not same."));

			}
		} else {
			throw (new IOException(
					"File " + fSrc.getAbsolutePath() + " or " + fCmp.getAbsolutePath() + " does not exist."));

		}

	}

	/**
	 * Compare Text File: Compare two text files and also consider or ignore case.
	 * 
	 * @param fSrcPath
	 *            is first text file path in string.
	 * @param fCmpPath
	 *            is second text file path in string.
	 * @param ignoreCase
	 *            is boolean value to/not to ignore case.
	 * @throws IOException
	 *             if file size not same or file dosen't exist.
	 * @return <code> boolean </code> as true if both files are same otherwise it
	 *         will return false.
	 */
	public static boolean compareTextFiles(String fSrcPath, String fCmpPath, boolean ignoreCase) throws IOException {
		File fSrc = new File(fSrcPath);
		File fCmp = new File(fCmpPath);

		if (fSrc.isFile() && fCmp.isFile()) {
			String fSrcStr = new String(Files.readAllBytes(fSrc.toPath()));
			String fCmpStr = new String(Files.readAllBytes(fCmp.toPath()));

			if (fSrcStr.length() == fCmpStr.length()) {
				if (ignoreCase) {
					return fSrcStr.equalsIgnoreCase(fCmpStr);

				} else {
					return fSrcStr.equals(fCmpStr);
				}
			} else {

				throw (new IOException("File size not same."));
			}
		} else {
			throw (new IOException("File " + fSrcPath + " or " + fCmpPath + " does not exist."));
		}

	}

	/**
	 * Search String using regular expression: Search text based on regular
	 * expression.
	 * 
	 * @param fSrcPath
	 *            is text file path in string.
	 * @param regrex
	 *            is regular expression string.
	 * @throws IOException
	 *             if file not found.
	 * @return <code> boolean</code> This method will return true if text within
	 *         file satisfies regular expression.
	 */
	public static boolean searchRegrex(String fSrcPath, String regrex) throws IOException {
		File fSrc = new File(fSrcPath);
		Pattern regrexPattern = Pattern.compile(regrex);
		String fSrcStr = new String(Files.readAllBytes(fSrc.toPath()));
		Matcher matcher = regrexPattern.matcher(fSrcStr);
		return matcher.find();
	}

	/**
	 * Search String using regular expression: Search text based on regular
	 * expression.
	 * 
	 * @param fSrc
	 *            is text file object.
	 * @param regrex
	 *            is regular expression string.
	 * @throws IOException
	 *             if file not found.
	 * @return <code> boolean</code> This method will return true if text within
	 *         file satisfies regular expression.
	 */
	public static boolean searchRegrex(File fSrc, String regrex) throws IOException {
		Pattern regrexPattern = Pattern.compile(regrex);
		String fSrcStr = new String(Files.readAllBytes(fSrc.toPath()));
		Matcher matcher = regrexPattern.matcher(fSrcStr);
		return matcher.find();
	}

	/**
	 * Replace String In text file: It will search text based on regular expression
	 * and replace all matching text from file.
	 * 
	 * @param fSrc
	 *            is text file object.
	 * @param regrex
	 *            is regular expression string.
	 * @param strReplace
	 *            is new string to be replaced with old one
	 * @throws IOException
	 *             if file not found.
	 */
	public static void replaceRegrex(File fSrc, String regrex, String strReplace) throws IOException {
		String fSrcStr = new String(Files.readAllBytes(fSrc.toPath()));
		String fStr = fSrcStr.replaceAll(regrex, strReplace);
		FileWriter fileWriter = new FileWriter(fSrc);
		fileWriter.write(fStr);
		fileWriter.close();
	}

	/**
	 * Replace String In text file: It will search text based on regular expression
	 * and replace all matching text from file.
	 * 
	 * @param fSrcPath
	 *            is text file path in string.
	 * @param regrex
	 *            is regular expression string.
	 * @param strReplace
	 *            is replacement string.
	 * @throws IOException
	 *             if file not found.
	 */
	public static void replaceRegrex(String fSrcPath, String regrex, String strReplace) throws IOException {
		File fSrc = new File(fSrcPath);
		String fSrcStr = new String(Files.readAllBytes(fSrc.toPath()));
		String fStr = fSrcStr.replaceAll(regrex, strReplace);
		FileWriter fileWriter = new FileWriter(fSrc);
		fileWriter.write(fStr);
		fileWriter.close();
	}

}
